@@ -14,41 +14,63 @@
 -- Further documentation and information can be found at
 -- <http://www.haskell.org/haskellwiki/Library/Data_encoding>.
 module Codec.Binary.QuotedPrintable
-    ( encode
+    ( EncIncData(..)
+    , EncIncRes(..)
+    , encodeInc
+    , encode
+    , DecIncData(..)
+    , DecIncRes(..)
+    , decodeInc
     , decode
-    , decode'
     , chop
     , unchop
     ) where
 
-import Data.Char
-import Data.Word
-
 import Codec.Binary.Util
 
--- {{{1 encode
--- | Encode data.
-encode :: [Word8]
-    -> String
-encode [] = ""
-encode (o : os)
-    | o < 33 || o == 61 || o > 126 = ('=' : toHex o) ++ encode os
-    | otherwise = chr (fromIntegral o) : encode os
+import Data.Char
+import Data.Maybe
+import Data.Word
 
+-- {{{1 encode
+-- | Incremental encoder function.
+encodeInc :: EncIncData -> EncIncRes String
+encodeInc e = eI e
+    where
+        enc [] = []
+        enc (o:os)
+            | o < 33 || o == 61 || o > 126 = ('=' : toHex o) ++ enc os
+            | otherwise = chr (fromIntegral o) : enc os
+
+        eI EDone = EFinal []
+        eI (EChunk bs) = EPart (enc bs) encodeInc
+
+-- | Encode data.
+encode :: [Word8] -> String
+encode = encoder encodeInc
 
 -- {{{1 decode
--- -- | Decode data (lazy).
-decode' :: String
-    -> [Maybe Word8]
-decode' [] = []
-decode' ('=' : c0 : c1 : cs) = fromHex [c0, c1] : decode' cs
-decode' (c : cs)
-    | c /= '=' = (Just $ fromIntegral $ ord c) : decode' cs
-    | otherwise = [Nothing]
+-- | Incremental decoder function.
+decodeInc :: DecIncData String -> DecIncRes String
+decodeInc d = dI [] d
+    where
+        dI [] DDone = DFinal [] []
+        dI lo DDone = DFail [] lo
+        dI lo (DChunk s) = doDec [] (lo ++ s)
+            where
+                doDec acc [] = DPart acc (dI [])
+                doDec acc s'@('=':c0:c1:cs) = let
+                        o = fromHex [c0, c1]
+                    in if isJust o
+                        then doDec (acc ++ [fromJust o]) cs
+                        else DFail acc s'
+                doDec acc s'@(c:cs)
+                    | c /= '=' = doDec (acc ++ [fromIntegral $ ord c]) cs
+                    | otherwise = DPart acc (dI s')
 
--- | Decode data (strict).
+-- | Decode data.
 decode :: String -> Maybe [Word8]
-decode = sequence . decode'
+decode = decoder decodeInc
 
 -- {{{1 chop
 -- | Chop up a string in parts.
